/******************************************************************************
 *
 *
 * Copyright AllSeen Alliance. All rights reserved.
 *
 *    Permission to use, copy, modify, and/or distribute this software for any
 *    purpose with or without fee is hereby granted, provided that the above
 *    copyright notice and this permission notice appear in all copies.
 *
 *    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 *    WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 *    MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 *    ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *    WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 *    ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ******************************************************************************/
#include <gtest/gtest.h>

#include <alljoyn/Init.h>
#include <alljoyn/Status.h>

#include <qcc/Debug.h>

#include <string.h>

static void DebugOut(DbgMsgType type, const char* module, const char* msg, void* context)
{
    QCC_UNUSED(type);
    QCC_UNUSED(module);
    QCC_UNUSED(msg);
    QCC_UNUSED(context);
    // Do nothing to suppress AJ errors and debug prints.
}

static bool IsDebugOn(char** env)
{
    while (env && *env) {
        if (strncmp(*env, "ER_DEBUG_", 9) == 0) {
            return true;
        }
        ++env;
    }
    return false;
}


/** Main entry point */
int CDECL_CALL main(int argc, char** argv, char** envArg)
{
    if (AllJoynInit() != ER_OK) {
        return 1;
    }
    if (AllJoynRouterInit() != ER_OK) {
        AllJoynShutdown();
        return 1;
    }

    int status = 0;
    setvbuf(stdout, NULL, _IONBF, 0);
    setvbuf(stderr, NULL, _IONBF, 0);

    if (!IsDebugOn(envArg)) {
        QCC_RegisterOutputCallback(DebugOut, NULL);
    }

    printf("\n Running alljoyn_core unit test\n");
    testing::InitGoogleTest(&argc, argv);
    status = RUN_ALL_TESTS();

    printf("%s exiting with status %d \n", argv[0], status);

    AllJoynRouterShutdown();
    AllJoynShutdown();
    return (int) status;
}
